@@ -1,6 +1,7 @@
 # Copyright (C) 2015 Sebastian Pipping <sebastian@pipping.org>
 # Licensed under GPL v2 or later
 
+import subprocess
 import sys
 
 from .parser import extract_flags
@@ -19,11 +20,13 @@ class Engine:
 
     @staticmethod
     def _extract_arch_from_flags(flags):
-        prefix = '-march='
-        for flag in flags:
-            if flag.startswith(prefix):
-                return flag[len(prefix):]
-        raise ValueError('No entry -march=.. found in: %s' %
+        for prefix in ('-march=', '-mcpu='):
+            for flag in flags:
+                if flag.startswith(prefix):
+                    arch = flag[len(prefix):]
+                    if arch:
+                        return arch
+        raise ValueError('No entry -m(arch|cpu)=.. found in: %s' %
                          ' '.join(sorted(flags)))
 
     @staticmethod
@@ -33,7 +36,7 @@ class Engine:
             if flag.startswith(prefix):
                 return (flag, flag[len(prefix):])
         raise NoTunePresentError(
-            'No entry -tune=.. found in: %s' % ' '.join(sorted(flags)))
+            'No entry -mtune=.. found in: %s' % ' '.join(sorted(flags)))
 
     @staticmethod
     def _dump_flags(flags):
@@ -54,29 +57,33 @@ class Engine:
                 flag_set.remove(flag)
 
     def _get_march_native_flag_set(self):
-        march_native_flag_set = set(extract_flags(
-            run(self._gcc_command, ['-march=native'], self._debug)))
+        try:
+            output = run(self._gcc_command, ['-march=native'], self._debug)
+        except subprocess.CalledProcessError:
+            output = run(self._gcc_command, ['-mcpu=native'], self._debug)
+        march_native_flag_set = set(extract_flags(output))
         if self._debug:
             self._dump_flags(march_native_flag_set)
-        march_native_flag_set |= set(get_flags_implied_by_march('native', gcc=self._gcc_command))
+        march_native_flag_set |= set(get_flags_implied_by_march('native', gcc=self._gcc_command,
+                                                                debug=self._debug))
         if self._debug:
             self._dump_flags(march_native_flag_set)
         return march_native_flag_set
 
     def _get_march_explicit_flag_set(self, arch):
-        march_explicit_flag_set = set(extract_flags(
-            run(self._gcc_command, [self._get_march_explicit(arch)], self._debug)))
+        try:
+            output = run(self._gcc_command, ['-march=%s' % arch], self._debug)
+        except subprocess.CalledProcessError:
+            output = run(self._gcc_command, ['-mcpu=%s' % arch], self._debug)
+        march_explicit_flag_set = set(extract_flags(output))
         if self._debug:
             self._dump_flags(march_explicit_flag_set)
-        march_explicit_flag_set |= set(get_flags_implied_by_march(arch, gcc=self._gcc_command))
+        march_explicit_flag_set |= set(get_flags_implied_by_march(arch, gcc=self._gcc_command,
+                                                                  debug=self._debug))
         if self._debug:
             self._dump_flags(march_explicit_flag_set)
         return march_explicit_flag_set
 
-    @staticmethod
-    def _get_march_explicit(arch):
-        return '-march=%s' % arch
-
     def _process_flags_explicit_has_more(self, target_set,
                                          march_native_flag_set,
                                          march_explicit_flag_set):
@@ -95,14 +102,18 @@ class Engine:
                 opposite_flag = PREFIX_NO + flag[len(PREFIX_YES):]
                 target_set.add(opposite_flag)
 
-    def _resolve(self, march_native_flag_set, march_explicit_flag_set, arch, options):
+    def _resolve(self, march_native_flag_set, march_explicit_flag_set, arch, options,
+                 march_wanted):
         if options.keep_identical_mtune:
             # NOTE: Set ``march_explicit_flag_set`` will be subtracted below and it may contain
             #       ``-mtune={arch}``.  So by removing mtune from the remove list, it survives.
             self._resolve_mtune(march_explicit_flag_set, arch)
 
         native_unrolled_flag_set = march_native_flag_set - march_explicit_flag_set
-        native_unrolled_flag_set.add(self._get_march_explicit(arch))
+        if march_wanted:
+            native_unrolled_flag_set.add('-march=%s' % arch)
+        else:
+            native_unrolled_flag_set.add('-mcpu=%s' % arch)
 
         if not options.keep_identical_mtune:
             self._resolve_mtune(native_unrolled_flag_set, arch)
@@ -118,8 +129,11 @@ class Engine:
 
     def run(self, options):
         march_native_flag_set = self._get_march_native_flag_set()
+        march_wanted = ('-march=' in repr(march_native_flag_set)
+                        and '-march=' not in march_native_flag_set)
         arch = self._extract_arch_from_flags(march_native_flag_set)
         march_explicit_flag_set = self._get_march_explicit_flag_set(
             arch)
 
-        return self._resolve(march_native_flag_set, march_explicit_flag_set, arch, options)
+        return self._resolve(march_native_flag_set, march_explicit_flag_set, arch, options,
+                             march_wanted)
